using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;


namespace getting_droped_frames
{
    public partial class Form1 : Form
    {

        // Save the ring buffer index of the last processed frame.
        private int LastHandledFrameIndex;

        public Form1()
        {
            LastHandledFrameIndex = 0;
            InitializeComponent();
        }

        private void Form1_Load(object sender, EventArgs e)
        {

            icImagingControl1.ShowDeviceSettingsDialog();

            if (icImagingControl1.DeviceValid )
            {
                icImagingControl1.LiveDisplayDefault = false;
                icImagingControl1.LiveDisplayHeight = icImagingControl1.Height;
                icImagingControl1.LiveDisplayWidth = icImagingControl1.Width;

                
                icImagingControl1.LiveDisplay = false;

                // To recieve dropped frames it is necessary to set
                // OverlayUpdateEventEnable to false.
                icImagingControl1.OverlayUpdateEventEnable = false;

                // Set the imagebuffer (must be large enough).
                icImagingControl1.ImageRingBufferSize = 60;
                icImagingControl1.LiveCaptureContinuous = true;
                icImagingControl1.LiveStart();

            }

        }

        private void IcImagingControl1_ImageAvailable(object sender, TIS.Imaging.ICImagingControl.ImageAvailableEventArgs e)
        {
            TIS.Imaging.ImageBuffer buffer = icImagingControl1.ImageBuffers[e.bufferIndex];

            // Calculate the difference between the last processed buffer index and the current buffer
            // index. This will return the count of dropped frames.
            int DroppedFramesCount = e.bufferIndex - LastHandledFrameIndex;

            // If DroppedFramesCount is less than zero then the ring buffer index has been started
            // again at zero between the LastHandledFrameIndex and then current e.bufferIndex. 
            if (DroppedFramesCount < 0)
                DroppedFramesCount = icImagingControl1.ImageRingBufferSize + DroppedFramesCount;

            // If frames have been dropped, then display all non processed frames in the following
            // loop.
            while (DroppedFramesCount > 1)
            {
                // Calculate the index of a dropped frame.
                int DroppedFrameIndex = LastHandledFrameIndex - DroppedFramesCount - 1;
                // Handle an overflow of the RingBufferIndex.
                if (DroppedFrameIndex < 0)
                {
                    //Keep in mind DroppedFrameIndex is negative!
                    DroppedFrameIndex = icImagingControl1.ImageRingBufferSize + DroppedFrameIndex;
                }

                // Get the dropped frame.
                TIS.Imaging.ImageBuffer LostFrame = icImagingControl1.ImageBuffers[DroppedFrameIndex];
                // Display the dropped frame.
                icImagingControl1.DisplayImageBuffer(LostFrame);

                DroppedFramesCount--;
            }

            //Sleep for simulate time expensive image processing.
            System.Threading.Thread.Sleep(300);

            icImagingControl1.DisplayImageBuffer(buffer);

            // Save the ring buffer index of the current frame into LastHandledFrameIndex. This will
            // be used next time ImageAvailable is called again. 
            LastHandledFrameIndex = e.bufferIndex;
        }



    }
}
